# package bmap ; 

# require Exporter ; 
# @ISA = ('Exporter') ;

# @EXPORT = qw(
# 	showlatlon
# 	panmap
# 	bindfindclosest
# 	bindpanmap
# 	zoommap
# 	bindzoommap
# 	bindenter
# 	bindstartdrag
# 	binddrag
# 	bindstopdrag
# ) ; 

# use Tk ;
# use Tk::After ;
# use Tk::Balloon ;
# use Tk::Canvas ;
# use Tk::Bplot ;

# use Datascope ;
# use Vector ;

our $Dragwindow ;

sub showlatlon {
    my($w, $latlon) = @_; 
    my ( $vp, $inside ) = bplot_locate ( $w ) ;
    #    if the mouse is not inside a plot region, clear the display
    if ( ! defined $inside || $inside == 0 ) { 
	    $$latlon  =  "" ;
    } else { 
	my $e = $w->XEvent; 
	my ($lon, $lat) = viewport_pixels2wcoords ( $vp, $e->x, $e->y ) ;
	if ($lat < 0.0) {
		$lat = sprintf "S%06.3f", -$lat ;
	} else {
		$lat = sprintf "N%06.3f", $lat ;
	}
	if ($lon < 0.0) {
		$lon = sprintf "W%07.3f", -$lon ;
	} else {
		$lon = sprintf "E%07.3f", $lon ;
	}
	$$latlon  = $lat . " " . $lon ;
    }
}

#    this procedure will move (pan) the edp viewport
#    map reference lat-lon to a point specified by
#    a ::Canvas coordinate

sub panmap {
	my $vp = shift ;
	my $x = shift ;
	my $y = shift ;

	my ($lon, $lat) = viewport_pixels2wcoords ( $vp, $x, $y ) ;

	if ( $vp eq "vp" ) {
		$::Canvas->itemconfigure ( $vp,
				-latr => $lat,
				-lonr => $lon
			) ;
	} else {
		my ($xl) = $::Canvas->itemcget ( $vp, -xleft ) ;
		my ($xr) = $::Canvas->itemcget ( $vp, -xright ) ;
		my ($yb) = $::Canvas->itemcget ( $vp, -ybottom ) ;
		my ($yt) = $::Canvas->itemcget ( $vp, -ytop ) ;

		my $lonr = ($xr-$xl) ;
		my $latr = ($yt-$yb) ;

		$xl = $lon - 0.5*$lonr ;
		$xr = $lon + 0.5*$lonr ;
		$yb = $lat - 0.5*$latr ;
		$yt = $lat + 0.5*$latr ;

		while ( $xr > 180.0 ) {
			$xl -= 360.0 ;
			$xr -= 360.0 ;
		}
		while ( $xl < -360.0 ) {
			$xl += 360.0 ;
			$xr += 360.0 ;
		}

		$::Canvas->itemconfigure ( $vp,
				-xleft => $xl,
				-xright => $xr,
				-ybottom => $yb,
				-ytop => $yt,
			) ;
	}
}

#    this procedure binds to left mouse clicks
#    and will find the closest event, warp the cursor
#    to the event and bring up a balloon that describes
#    the event

sub bindfindclosest {
    my($w) = @_; 
    my ( $vp, $inside, @entries ) = bplot_locate ( $w, "-getentries" ) ;
    #    $inside is set if the event was inside of the viewport plot area
    my (@entry, $index, $item, $v) ; 
    my ($lon, $lat, $label) ;
    my ($ref, @db, $msg) ;
    if ( defined $inside && $inside != 0 ) { 
	foreach my $entry (@entries) { 
	    next if $entry eq "" ;
	    @entry = split /\s+/, $entry ;
	    $index = $entry[1] ;
	    $item = $entry[0] ;
	    ($v) = $w->itemcget($item, -vector) ; 
	    last if $v eq $Show_this_vector ; 
	}
	if ($v eq $Show_this_vector) { 
	    $ref = $Db{$v} ;
	    @db = @$ref ;
	    $db[3] = $index ;
	    $msg = show_record(@db) ;
	    ($lon, $lat, $label) = vector_get_point ( $v, $index );
	    my ($x, $y) = viewport_wcoords2pixels ( $vp, $lon, $lat ) ;
	    $w->eventGenerate ( '<Motion>',
			    -warp => 1,
			    -x => $x,
			    -y => $y,
		    ) ;

	    #    get event info and attach the balloon

	    foreach my $id ( $w->afterInfo () ) {
		    $w->afterCancel ( $id ) ;
	    }
	    #    set balloon position and attach balloon
	    my $pos = sprintf "%d,%d", $w->rootx+$x-10, $w->rooty+$y-10 ;
	    $Balloon->attach ( $w, 
			    -msg => $msg, 
			    -postcommand => sub { $pos ; },
			    -motioncommand => sub { 0 ; },
		    ) ;
	    #    this bit causes the balloon to "expire" after 5 seconds
	    $w->after ( $Linger, sub { $Balloon->detach ( $w ) ; } ) ;
	}
    }
}

sub bindpanmap {
	my($w) = @_; 
	my $e = $w->XEvent; 
	my ( $vp, $inside ) = bplot_locate ( $w ) ;
	if ( defined $inside && $inside == 1 ) {panmap ( $vp, $e->x, $e->y ) ;}
}

#    this procedure will zoom in/out either of the 
#    map viewports according to a factor. The x y
#    canvas coordinate is used to determine which
#    viewport to zoom.

sub zoommap {
    my $vp = shift ;
    my $x = shift ;
    my $y = shift ;
    my $factor = shift ;

    my ($xl) = $::Canvas->itemcget ( $vp, -xleft ) ;
    my ($xr) = $::Canvas->itemcget ( $vp, -xright ) ;
    my ($yb) = $::Canvas->itemcget ( $vp, -ybottom ) ;
    my ($yt) = $::Canvas->itemcget ( $vp, -ytop ) ;

    my $xc = 0.5*($xl+$xr) ;
    my $yc = 0.5*($yb+$yt) ;

    $xl -= $xc ;
    $xr -= $xc ;
    $yb -= $yc ;
    $yt -= $yc ;

    $xl = $xc + $factor * $xl ;
    $xr = $xc + $factor * $xr ;
    $yb = $yc + $factor * $yb ;
    $yt = $yc + $factor * $yt ;

    $::Canvas->itemconfigure ( $vp,
		    -xleft => $xl,
		    -xright => $xr,
		    -ybottom => $yb,
		    -ytop => $yt,
	    ) ;
}

sub bindzoommap {
    my($w) = @_; 
    my $e = $w->XEvent; 
    my ( $vp, $inside ) = bplot_locate ( $w ) ;
    if ( ! defined $inside ) {return;}
    if ( $inside == 0 ) {return;}
    my $key = $e->K ;
    if ( $key eq "O" ) { zoommap ($vp, $e->x, $e->y, 2.0 ) ; }
    if ( $key eq "o" ) { zoommap ($vp, $e->x, $e->y, 1.25 ) ; }
    if ( $key eq "I" ) { zoommap ($vp, $e->x, $e->y, 0.5 ) ; }
    if ( $key eq "i" ) { zoommap ($vp, $e->x, $e->y, 0.8 ) ; }
}

sub bindenter {
    my($w) = @_; 
    my $e = $w->XEvent; 
    $::Canvas->CanvasFocus () ;
}

sub bindstartdrag {
    my ($w) = @_;
    my ( $vp, $inside ) = bplot_locate ( $w ) ;
    if ( defined $vp && $inside != 0 ) { 
	$Dragwindow = $vp ;
	my $e = $w->XEvent ;
	our ($xstart, $ystart) = ( $e->x, $e->y ) ;
    }
}

sub binddrag {
    my ($w) = @_;
    if ( defined $Dragwindow ) { 
	my $e = $w->XEvent ;
	my ($x, $y) = ( $e->x, $e->y ) ;
	our ($xstart, $ystart) ;
	my $delx = $x - $xstart;
	my $dely = $y - $ystart;
	$w->itemconfigure ( $Dragwindow,
				-xtranslate => -$delx,
				-ytranslate => -$dely,
			) ;
    }
}

sub bindstopdrag {
    my ($w) = @_;
    if ( defined $Dragwindow ) { 
	$w->itemconfigure ( $Dragwindow,
				-xtranslate => 'apply',
				-ytranslate => 'apply',
			) ;
	undef $Dragwindow ;
    }
}

1;
